# Generated by Django 1.10.7 on 2017-06-08 06:57

from django.db import migrations, models

from django.utils import timezone


class Migration(migrations.Migration):
    dependencies = [
        ('social_django', '0007_code_timestamp'),
    ]

    operations = [
        migrations.AddField(
            model_name='partial',
            name='timestamp',
            field=models.DateTimeField(auto_now_add=True,
                                       db_index=True,
                                       default=timezone.now),
            preserve_default=False,
        ),
    ]
